from __future__ import absolute_import
"""This package contains files pulled from other projects to avoid
dependning on an entire project to get a single utility function."""
from .pyramid_decorator import reify

__all__ = ['reify']
